% "A Theory of Intrinsic Inflation Persistence" 
% by Takushi Kurozumi and Willem Van Zandweghe
% Figure 7
%% Calibration
beta = 0.99;
sigman = 1;
epsp = -3;
epsw = -3;
thetap = 10;
thetaw = 10;
alpp = 0.75;
alpw = 0;
phip = 1.5;
phiy = 0.125;
rho1 = 0;
rho_i = 0; 
std_i = 0.0025; 
rho_a = 0;
std_a = 0;
shocks = [rho_i std_i; rho_a std_a];

%% Calculate responses to a disinflation
% Baseline model (no indexation)
    [dis_p,dis_y] = disinflation(beta,epsp,epsw,thetap,thetaw,alpp,alpw,phip,phiy,rho1,shocks);
% Indexation to past inflation
    iotap = 0.24;
    iotaw = 0.58;
    [dispast_p,dispast_y] = disinflation_indexation(iotap,iotaw,beta,sigman,epsp,epsw,thetap,thetaw,alpp,alpw,phip,phiy,rho1,shocks);
% Indexation to trend inflation
    iotap = 0;
    iotaw = 0;
    [distrend_p,distrend_y] = disinflation_indexation(iotap,iotaw,beta,sigman,epsp,epsw,thetap,thetaw,alpp,alpw,phip,phiy,rho1,shocks);

%% Plot responses to a one-standard-deviation accommodative shock
tmax = 20;
time = (0:tmax);
xaxis = [0 0.25 0.5 0.75 1]*tmax;
fig = figure;
left_color = [0 0 0];
right_color = [0 0 0];
set(fig,'defaultAxesColorOrder',[left_color; right_color]);
subplot(1,2,1)
    yyaxis left
    hl = plot(time,400*dis_p);
    set(gca,'FontSize',7)
    set(hl,'LineWidth',1.2,'LineStyle','-','Color',[0 0 1]);
    set(gca,'YLim',[-5 0.2],'YTick',[-5 -4 -3 -2 -1 0])
    set(gca,'FontSize',7)
    ylabel('pp deviation from initial ss')
    xlabel('Quarters')
    yyaxis right
    hr = plot(time,400*[distrend_p; dispast_p]);
    set(hr(1),'LineWidth',1.2,'LineStyle','--','Color',[0 0 0]);
    set(hr(2),'LineWidth',1.2,'LineStyle',':','Color',[0 0 0]);
    set(gca,'YLim',[-5 0.2],'YTick',[-5 -4 -3 -2 -1 0])
    set(gca,'XLim',[0 16],'XTick',[0 4 8 12 16],'XTickLabel',[0 4 8 12 16])
    set(gca,'FontSize',7)
    xlabel('Quarters')
	title('Inflation')
subplot(1,2,2)
    yyaxis left
    hl = plot(time,100*dis_y);
    set(hl,'LineWidth',1.2,'LineStyle','-','Color',[0 0 1]);
    set(gca,'YLim',[-2.4 0.3],'YTick',[-2.4 -1.8 -1.2 -0.6 0])
    set(gca,'FontSize',7)
    ylabel('% deviation from initial ss')
    yyaxis right
    hr = plot(time,100*[distrend_y; dispast_y]);
    set(hr(1),'LineWidth',1.2,'LineStyle','--','Color',[0 0 0]);
    set(hr(2),'LineWidth',1.2,'LineStyle',':','Color',[0 0 0]);
    set(gca,'YLim',[-0.4 0.05],'YTick',[-0.4 -0.3 -0.2 -0.1 0])
    set(gca,'XLim',[0 16],'XTick',[0 4 8 12 16],'XTickLabel',[0 4 8 12 16])
    set(gca,'FontSize',7)
    xlabel('Quarters')
	title('Output')
    h = legend([ hl hr(2) hr(1) ],'No indexation (L)',...
        'Index. to past inflation (R)',...
        'Index. to trend inflation (R)','Location','East','FontSize',6);
    set(h,'Interpreter','latex')
    
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperSize', [5 2.2 ]);
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperPosition', [-0.3 0.1 5.6 2 ]);
set(gcf, 'renderer', 'painters');
print(gcf, '-dpdf', 'fig7_disinflation.pdf');

%% Functions
function [dis_p,dis_y] = disinflation(beta,epsp,epsw,thetap,thetaw,alpp,alpw,phip,phiy,rho1,shocks)
    % Steady-state values of the endogenous state variables
    % In period 0, before disinflation is announced
    bigpi0=1.067^0.25;
    [Y0,Delta0,dp0,dw0,w0,i0] = atiip_steady_state(bigpi0,beta,epsp,epsw,thetap,thetaw,alpp,alpw);
    if not(imag(Y0)==0)
        error('Calibration violates an assumption for the existence of a steady state')
    end
    % In period 1, when disinflation is announced
    bigpi1=1.02^0.25;
    [Y1,Delta1,dp1,dw1,w1,i1] = atiip_steady_state(bigpi1,beta,epsp,epsw,thetap,thetaw,alpp,alpw);
    if not(imag(Y1)==0)
        error('Calibration violates an assumption for the existence of a steady state')
    end
    
    % Decision rules
    [G1,~,~,eu] = ATIIP_fun(bigpi1,beta,epsp,epsw,thetap,thetaw,alpp,alpw,phip,phiy,0,rho1,shocks,0);
    if sum(eu)<2; disp('No unique solution'); end
    % Gensys solution takes form y(t) = G1*y(t-1)
    % find position of endogenous state variables in the columns of G1
    nx = size(G1,2);
    V = zeros(1,nx);
    for i=1:nx
        if max(G1(:,i)) > 1e-8
            V(i) = 1;
        end
    end
    j = find(V==1);
    G2 = G1(:,j);
    % The lagged endog. state variables in period 1 are equal to the log of their 
    % period-0 steady-state values minus the log of their period-1 steady-state values 
    if alpw>0
        kss0 = log([Delta0;dp0;dw0;w0]);
        kss1 = log([Delta1;dp1;dw1;w1]);
    elseif alpw==0
        kss0 = log([Delta0;dp0]);
        kss1 = log([Delta1;dp1]);
    end
    dkss = kss0-kss1;
    
    % Credible disinflation
    horizon = 20;
    dis = zeros(nx,horizon);
    
    dk = dkss;
    for i = 1:horizon
        dis(:,i) = G2*dk;
        dk = dis(j,i);
    end
    
    % convert to pct deviations from initial steady state
    dis(1,:) = dis(1,:) + log(bigpi1) - log(bigpi0); % inflation
    dis_p = [ 0 dis(1,:)];
    if alpw>0
        dis(10,:) = dis(10,:) + log(Y1) - log(Y0); % output
        dis_y = [ 0 dis(10,:)];
    elseif alpw==0
        dis(8,:) = dis(8,:) + log(Y1) - log(Y0); % output
        dis_y = [ 0 dis(8,:)];
    end
end
function [dis_p,dis_y] = disinflation_indexation(iotap,iotaw,beta,sigman,epsp,epsw,thetap,thetaw,alpp,alpw,phip,phiy,rho1,shocks)
    % Steady-state values of the endogenous state variables
    % In period 0, before disinflation is announced
    bigpi0=1.067^0.25;
    mc0 = (thetap-1)/thetap;
    Y0 = (mc0*(thetaw-1)/thetaw)^(1/(1+sigman));
    i0 = bigpi0/beta;
    % In period 1, when disinflation is announced
    bigpi1=1.02^0.25;
    mc1 = mc0;
    Y1 = Y0;
    i1 = bigpi1/beta;
    
    % Decision rules
    [G1,~,eu] = ATIIP_indexation_fun(iotap,iotaw,beta,sigman,epsp,epsw,thetap,thetaw,alpp,alpw,phip,phiy,0,rho1,shocks(1,2),0);
    if sum(eu)<2; disp('No unique solution'); end
    % Gensys solution takes form y(t) = G1*y(t-1)
    % find position of endogenous state variables in the columns of G1
    nx = length(G1);
    V = zeros(1,nx);
    for i=1:nx
        if max(G1(:,i)) > 1e-8
            V(i) = 1;
        end
    end
    j = find(V==1);
    G2 = G1(:,j);
    % Case of no endogenous state variables
    sv = isempty(G2);
    if sv==1
        G2=zeros(nx,1);
        j=1;
    end
    % The lagged endog. state variables in period 1 are equal to the log of their 
    % period-0 steady-state values minus the log of their period-1 steady-state values 
    if alpw>0
        if or(iotap>0,iotaw>0)
            kss0 = log([bigpi0;mc0]);
            kss1 = log([bigpi1;mc1]);
        elseif and(iotap==0,iotaw==0)
            kss0 = log(mc0);
            kss1 = log(mc1);
        end
    elseif alpw==0
        if or(iotap>0,iotaw>0)
            kss0 = log(bigpi0);
            kss1 = log(bigpi1);
        elseif and(iotap==0,iotaw==0)
            kss0 = 0;
            kss1 = 0;
        end
    end
    
    dkss = kss0-kss1;
    
    % Credible disinflation
    horizon = 20;
    dis = zeros(nx,horizon);
    
    dk = dkss;
    for i = 1:horizon
        dis(:,i) = G2*dk;
        dk = dis(j,i);
    end
    
    % convert to pct deviations from initial steady state
    dis(1,:) = dis(1,:) + log(bigpi1) - log(bigpi0); % inflation
    dis_p = [ 0 dis(1,:)];
    if alpw>0
        dis(6,:) = dis(6,:) + log(Y1) - log(Y0); % output
        dis_y = [ 0 dis(6,:)];
    elseif alpw==0
        dis(5,:) = dis(5,:) + log(Y1) - log(Y0); % output
        dis_y = [ 0 dis(5,:)];
    end
end
function [Y,Delta,dp,dw,mc,i] = atiip_steady_state(bigpi,beta,epsp,epsw,thetap,thetaw,alpp,alpw)
    % Function calculates steady-state values of the endogenous state
    % variables in the baseline model 
    gamp=thetap*(1+epsp);
    s = (1-alpp)/(1-alpp*bigpi^gamp)*((1-alpp*bigpi^(gamp-1))/(1-alpp))^(gamp/(gamp-1));
    Delta = (s+epsp)/(1+epsp);
    
    p0 = ( 1/(1+epsp)*((1-alpp)/(1-alpp*bigpi^(gamp-1)))^(1/(1-gamp)) + epsp/(1+epsp)*(1-alpp)/(1-alpp*bigpi^(-1)) )^(-1);
    dp = ((1-alpp)/(1-alpp*bigpi^(gamp-1)))^(1/(1-gamp))*p0;
    
    eps1 = epsp*((1-alpp)/(1-alpp*bigpi^(gamp-1)))^(gamp/(gamp-1));
    eps2 = eps1*(1-alpp*beta*bigpi^(gamp-1))/(1-alpp*beta*bigpi^(-1));
    mc = ((gamp-1-eps2)/gamp)*(1-alpp*beta*bigpi^gamp)/(1-alpp*beta*bigpi^(gamp-1))*p0;
    
    psiY = 0; 
    gamw=thetaw*(1+epsw);
    w0 = ( 1/(1+epsw)*((1-alpw)/(1-alpw*bigpi^(gamw-1)))^(1/(1-gamw)) + epsw/(1+epsw)*(1-alpw)/(1-alpw*bigpi^(-1)) )^(-1);
    dw = ((1-alpw)/(1-alpw*bigpi^(gamw-1)))^(1/(1-gamw)) * w0;
    
    eps1w = epsw*( (1-alpw)/(1-alpw*bigpi^(gamw-1)) )^(gamw/(gamw-1));
    eps2w = eps1w*(1-alpw*beta*bigpi^(gamw-1)) / (1-alpw*beta*bigpi^(-1));
    eps3w = eps1w*(1-alpw*beta*bigpi^(2*gamw))/(1-alpw*beta*bigpi^gamw);
    Y = ( (gamw-1-eps2w)/gamw * (1-alpw*beta*bigpi^(2*gamw))/(1-alpw*beta*bigpi^(gamw-1)) * w0 * mc * (Delta+psiY)^(-1) *...
        (1+epsw)/(1+eps3w) * (w0/dw)^gamw )^(1/2);
    
    i = bigpi/beta;
end