function [v,P]=Household(x,T)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [v,P]=Household(x,T)
% This function, which is based on an extension of the Algorithm 5.5.1 of Golub and Van Loan, 'Matrix Computations',
% III Edition, p. 210, computes the Household vector v, and the Householder matrix P, for a previously defined 
% vector x (nxn). The Householder matrix P is such that:
%
% if T=='U'     ==>      P*x=[# zeros(1,n-1)]'
% if T=='L'     ==>      P*x=[zeros(1,n-1) #]'
%
% namely, if T=='U', the vector x is transformed into a vector such that all of the entries except for the first one have been
% zeroed out, while if T=='L' the vector x is transformed into a vector such that all of the entries except for the last one have
% been zeroed out.
%                                                           References
%
% Golub, G., and Van Loan, C. (1996), 'Matrix Computations', III edition
%
n=length(x);
%
if T=='U'
    sigma=x(2:n)'*x(2:n);
    v=[1 x(2:n)']';
    if sigma==0
        b=0;
    else
        mu=sqrt(x(1)^2+sigma); 
        if x(1)<=0
            v(1)=x(1)-mu;
        else
            v(1)=-sigma/(x(1)+mu);
        end
        b=(2*v(1)^2)/(sigma+v(1)^2);
        v=v/v(1);
    end
    P=eye(n)-b*v*v';
elseif T=='L'
    sigma=x(1:n-1)'*x(1:n-1);
    v=[x(1:n-1)' 1]';
    if sigma==0
        b=0;
    else
        mu=sqrt(x(n)^2+sigma); 
        if x(n)<=0
            v(n)=x(n)-mu;
        else
            v(n)=-sigma/(x(n)+mu);
        end
        b=(2*v(n)^2)/(sigma+v(n)^2);
        v=v/v(n);
    end
    P=eye(n)-b*v*v';
else
    disp('You did not specify which element of the vector x has to be kept: default choice of x(1) is used')
    sigma=x(2:n)'*x(2:n);
    v=[1 x(2:n)']';
    if sigma==0
        b=0;
    else
        mu=sqrt(x(1)^2+sigma); 
        if x(1)<=0
            v(1)=x(1)-mu;
        else
            v(1)=-sigma/(x(1)+mu);
        end
        b=(2*v(1)^2)/(sigma+v(1)^2);
        v=v/v(1);
    end
    P=eye(n)-b*v*v';
end



