function [Alpha,CSI0,CSI,A,B,E,Omega,MaxLogLikelihood,TraceStatistic,TestStatistic_h_versus_hplus1,PI0,PI]=JohansenCointegratedVAREstimates(Y,LagOrder)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [Alpha,CSI0,CSI,A,B,E,Omega,MaxLogLikelihood,TraceStatistic,TestStatistic_h_versus_hplus1]=JohansenCointegratedVAREstimates(Y,LagOrder)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   The program is based on the formulas found in Hamilton, Time Series Analysis, Chapter 20.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                       Input of the program is:
%  Y = a (TxN) vector of time series
%  LagOrder = the lag order for the cointegrated VAR
%
%                                                       Output of the program is:
%
% Important point: Alpha, CSI0, CSI, A, E, and Omega are computed for each possible value of the number of cointegrating vectors from 1 to N.
%
%  Alpha = the estimated intercept vector in the cointegrated VAR
%  CSI0  = the error-correction term in the VECM representation
%  CSI   = the VAR matrices in the VECM representation
%  A     = the matrix with the cointegrating vectors in the columns
%  B     = the matrix with the loadings
%  E     = the VAR's residuals
%  Omega = the covariance matrix of the VAR's residuals
%  MaxLogLikelihood = the maximised log-likelihoods subject to the constraints of h cointegrating relationships, for h = 1, 2, 3..., N
%  TraceStatistic   = Johansen's trace statistic
%  TestStatistic_h_versus_hplus1 = expression [20.3.7] in Hamilton
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Important: The way the various objects are constructed, we have, for zz = 1, 2, ... , N,
%
%                           CSI0(:,:,zz) = - B(:,1:zz)*A(:,1:zz)'
%
% (see equation [20.2.2] in Hamilton's book)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[T,N]=size(Y);
DY=diff(Y);
LagOrder_1=LagOrder-1;
TDY=size(DY,1);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                               CASE I: h cointegrating relationships and N-h deterministic time trends
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                   Step I: calculate auxiliary regressions
% Calculate a VAR(LagOrder-1) for DY(t):
[PI,VARB,U,V,vechV,VARvechV]=varp(DY,LagOrder_1,'Y',1,'NOR');
PI0=PI(:,1);
PI=PI(:,2:size(PI,2));
PI=reshape(PI,N,N,LagOrder_1);
% Calculate an OLS regression of Y(t-1) on DY(t-1) ... DY(t-(LagOrder+1)):
ZZ=ones(1,TDY-LagOrder_1);
for kk=1:LagOrder_1
    ZZ=[ZZ; DY(LagOrder_1+1-kk:TDY-kk,:)'];
end
YY=Y(LagOrder:T-1,:)';
% OLS estimator:
BB=(YY*ZZ')*MyInverse(ZZ*ZZ');
% Residuals:
V=YY-BB*ZZ;
Theta=BB(:,1);
BB=BB(:,2:size(BB,2));
CHI=reshape(BB,N,N,LagOrder-1);
TU=size(U,2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                   Step II: calculate canonical correlations
EUU=zeros(N);
EVV=zeros(N);
EUV=zeros(N);
for tt=1:TU
    EVV=EVV+V(:,tt)*V(:,tt)';
    EUU=EUU+U(:,tt)*U(:,tt)';
    EUV=EUV+U(:,tt)*V(:,tt)';
end
EVV=EVV/TU;
EUU=EUU/TU;
EUV=EUV/TU;
EVU=EUV';
% Find eigenvalues of (EVV\EVU)*(EUU\EUV):
[VV,DD]=OrderedEig((EVV\EVU)*(EUU\EUV));
Eigenvalues=diag(DD);
% Maximum value of the log-likelihood subject to the constraint that there are h cointegrating relationships:
MaxLogLikelihood=zeros(N,1);
TraceStatistic=zeros(N,1);
TestStatistic_h_versus_hplus1=zeros(N,1);
for hh=1:N
    MaxLogLikelihood(hh)=-(T*N/2)*log(2*pi)-(T*N/2)-(T/2)*log(det(EUU))-(T/2)*sum(log(-Eigenvalues(1:hh)+1));
    % The trace statistic: test of the null hypothesis that there are
    % at most (hh-1) cointegrating vectors (equation [20.3.7]) in Hamilton:
    TraceStatistic(hh)=-T*sum(log(-Eigenvalues(hh:N)+1));
    % Testing the null of h cointegrating vectors against the alternative
    % of h+1 cointegrating vectors (equation [20.3.7]) in Hamilton:
    TestStatistic_h_versus_hplus1(hh)=-T*log(1-Eigenvalues(hh));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           Step III: calculate maximum likelihood estimates of the parameters
for zz=1:N
    VV(:,zz)=VV(:,zz)/sqrt(VV(:,zz)'*EVV*VV(:,zz));
end
%
CSI0=zeros(N,N,N);
CSI=zeros(N,N,size(PI,3),N);
Alpha=zeros(N,1,N);
Omega=zeros(N,N,N);
for hh=1:N
    % The matrix of the cointegrating vectors:
    A=VV(:,1:hh); %
    % The MLE estimates:
    CSI0(:,:,hh)=EUV*A*A';
    for zz=1:size(PI,3)
        CSI(:,:,zz,hh)=PI(:,:,zz)-CSI0(:,:,hh)*CHI(:,:,zz);
    end
    Alpha(:,1,hh)=PI0-CSI0(:,:,hh)*Theta;
    for tt=1:TU
        Z=U(:,tt)-CSI0(:,:,hh)*V(:,tt);
        Omega(:,:,hh)=Omega(:,:,hh)+Z*Z';
    end
    Omega(:,:,hh)=Omega(:,:,hh)/TU;
end
A=VV; % The potential cointegrating vectors
B=-EUV*A;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                           Here we get the residuals from the cointegrated VAR:
YY=DY(LagOrder:TDY,:)';
ZZ=zeros(size(YY,1),size(YY,2),LagOrder_1);
for kk=1:LagOrder_1
    ZZ(:,:,kk)=DY(LagOrder_1+1-kk:TDY-kk,:)';
end
E=zeros(N,T-LagOrder,N);
for hh=1:N
    E(:,:,hh)=YY-CSI0(:,:,hh)*Y(LagOrder:T-1,:)';
    for tt=1:TU
        E(:,tt,hh)=E(:,tt,hh)-Alpha(:,:,hh);
    end
    for zz=1:LagOrder_1
        E(:,:,hh)=E(:,:,hh)-CSI(:,:,zz,hh)*ZZ(:,:,zz);
    end
end







