function [B,VARB,U,V,vechV,VARvechV]=varp(X,LagOrder,INT,HOR,HET)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [B,VARB,U,V,vechV,VARvechV]=varp(X,LagOrder,INT,HOR,HET)
% This program estimates a VAR(LagOrder) for K series, stored in the matrix X. It is based on the formulas in Lutkepohl,
% "Introduction to Multiple Time Series Analysis"
%
%                                             Input of the program is:
%
% X   = a matrix containing the K series stacked in columns: first column, the first series, second column, the second
%       series, etc.
% LagOrder   = the order of the VAR
% INT = 'Y' if you want an intercept vector, 'N' if you don't want it
% HOR = horizon (if missing it is set to one)
% HET = 'NOR' (no correction for heteroskedasticity or autocorrelation) or 'NEW' (Newey-West correction for heteroskedasticity or autocorrelation)
%
%                                             Output of the program is:
%
% B    = the OLS estimator, with B = [v A(1) A(2) ... A(LagOrder)] (see Lutkepohl)
% VARB = the variance covariance matrix for vec(B').
% U    = the residuals, stacked in columns: first column, the first series, second column, the second series, etc.
% V    = the variance-covariance matrix for the residuals U
% vechV    = vech(V)
% VARvechV = the variance-covariance matrix of vechV
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if nargin<4
    HOR=1;
    HET='NOR';
end
if nargin<5
    HET='NOR';
end
%
[T,K]=size(X);
%
Y=X(LagOrder+1+(HOR-1):T,:)';
%
if INT=='Y'
    Z=ones(1,T-LagOrder-(HOR-1));
else
    Z=[];
end
for kk=1:LagOrder
    Z=[Z; X(LagOrder+1-kk:T-(HOR-1)-kk,:)'];
end
%
% OLS estimator:
if INT=='Y'
    InvZZprime=(Z*Z')\eye(1+LagOrder*K);
else
    InvZZprime=(Z*Z')\eye(LagOrder*K);
end
%
B=(Y*Z')*MyInverse(Z*Z'); % Expression (3.2.10) in Lutkepohl
%
% Residuals:
U=Y-B*Z;
% Variance-covariance matrix:
V=(U*U')/(T-K*LagOrder-1);  % Expression (3.2.19) in Lutkepohl
%
Q=(Z*Z')/T;
% Variance-covariance matrix of b = vec(B'):
if HET=='NOR'
    VARB=kron(V,MyInverse(Q))/sqrt(T); % Expression (3.2.16) in Lutkepohl
else % The Newey-West correction
    U=U';
end
% Equivalent expression for vec(B'):
% kron(eye(K),(Z*Z')\Z)*vec(Y') % Expression (3.2.12) in Lutkepohl
% Variance covariance matrix for vech(V):
[vechV,HH,KK]=vech(V);
for hh=1:length(vechV)
    ii=HH(hh);
    jj=KK(hh);
    for kk=1:length(vechV)
        ll=HH(kk);
        mm=KK(kk);
        S22(hh,kk)=V(ii,ll)*V(jj,mm)+V(ii,mm)*V(jj,ll);
    end
end
VARvechV=S22/(T-K*LagOrder-1);  % Proposition (11.2) in Hamilton (1994), modified for the degrees of freedom.



