% "A Theory of Intrinsic Inflation Persistence" 
% by Takushi Kurozumi and Willem Van Zandweghe
% Table 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
clc
close all
delete *.asv
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Measures of inflation persistence for the US GDP deflator 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
path('table2files',path);
run EmpiricalPersistence.m

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Measures of inflation persistence for the baseline model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Model calibration and solution
beta = 0.99;
epsp = -3;
epsw = -3;
thetap = 10;
thetaw = 10;
alpp = 0.75;
alpw = 0.75;
phip = 1.5;
phiy = 0.125;
phidy = 0;
rho1 = 0.8;
rho_i = 0; 
rho_a = 0;
std_i = 0.0025;
std_a = 0;
shocks = [rho_i std_i; rho_a std_a];

pi_h = 1.067^0.25; %GDP deflator sample average 1970s
    [G1_h,impact_h,~,eu,kappa_h] = ATIIP_fun(pi_h,beta,epsp,epsw,thetap,thetaw,alpp,alpw,phip,phiy,phidy,rho1,shocks,0);
    if sum(eu)<2; disp('No unique solution'); end
pi_l = 1.02^0.25; 
    [G1_l,impact_l,irf,eu,kappa_l] = ATIIP_fun(pi_l,beta,epsp,epsw,thetap,thetaw,alpp,alpw,phip,phiy,phidy,rho1,shocks,0);
    if sum(eu)<2; disp('No unique solution'); end

%% Simulate the model
rng('default')
[Kx,Ku] = size(impact_h);
T = 10000; 
T0 = .1*T; %T0 first quarters to be discarded
Xh = zeros(Kx,T+T0);
Xl = zeros(Kx,T+T0);
Usimm = std_i.*randn(T+T0,1);
Usima = std_a.*randn(T+T0,1);
Usim = zeros(Ku,T+T0);
Usim(1,:) = Usimm;
if Ku==2
    Usim(2,:) = Usima;
end

Xh(:,1) = impact_h*Usim(:,1);
Xl(:,1) = impact_l*Usim(:,1);
for i = 2:T+T0
    Xh(:,i) = G1_h*Xh(:,i-1) + impact_h*Usim(:,i);
    Xl(:,i) = G1_l*Xl(:,i-1) + impact_l*Usim(:,i);
end
Xh = Xh(:,T0+1:end);
Xl = Xl(:,T0+1:end);

%% Sum of AR(p) coefficients (Fuhrer, 2011, Section 2.2.5)
p = 4; 
infl_h = Xh(1,:)';
infl_l = Xl(1,:)';
infl_h_p = zeros(T-p,p);
infl_l_p = zeros(T-p,p);
y_h = infl_h(1+p:end);
y_l = infl_l(1+p:end);
for i=1:p
    x_h = infl_h(1+p-i:T-i);
    infl_h_p(:,i) = x_h;
    x_l = infl_l(1+p-i:T-i);
    infl_l_p(:,i) = x_l;
end
statsout_h = regstats(y_h,infl_h_p);
statsout_l = regstats(y_l,infl_l_p);
beta_h = statsout_h.beta;
beta_l = statsout_l.beta;
sum_h = sum(beta_h(2:end));
sum_l = sum(beta_l(2:end));

%% Predictability (Cogley, Primiceri, and Sargent (AEJ Macro 2010)
[Rsq1_h,Rsq4_h,Rsq8_h] = predictabilityfun(G1_h,impact_h,std_i,std_a);
[Rsq1_l,Rsq4_l,Rsq8_l] = predictabilityfun(G1_l,impact_l,std_i,std_a);

%% Table 2, lower panel
tablename = {'Table 2: Model'};
rownames = {'pibar=6.7%','pibar=2%','Difference'};
ModRes = table(zeros(size(rownames,2),1),zeros(size(rownames,2),1),zeros(size(rownames,2),1), zeros(size(rownames,2),1),'VariableNames',{'Sum AR(p)','R²₁','R²₄','R²₈'},'RowNames',rownames);
ModRes = table(ModRes,'VariableNames',tablename);
ModRes{1,:} = {sum_h,Rsq1_h,Rsq4_h,Rsq8_h};
ModRes{2,:} = {sum_l,Rsq1_l,Rsq4_l,Rsq8_l};
ModRes{size(data,2)+1,:} = num2cell(table2array(ModRes{size(data,2),:}) - table2array(ModRes{size(data,2)-1,:}));
disp(ModRes)

function [Rsq1,Rsq4,Rsq8] = predictabilityfun(G1,impact,std_i,std_a)
    %=========================================================================
    % s_t = Phi*s(t-1) + R*e(t)
    Phi = G1;
    R = impact;
    nx = size(G1,1);
    Se = (100*std_i)^2;
    if std_a>0
        Se = [(100*std_i)^2, 0; 0, (100*std_a)^2];
    end
    % Approximate unconditional variance
    RSe = R*Se*R'; 
    %=========================================================================
    %       Predictability measure of inflation persistence
    %=========================================================================
    % Approximate forecast error variance, CPS (2010, eq. 10)
    % horizon h = 1,4,8
    h = 1;
    Sigs1 = zeros(nx,nx);
    for j=0:h-1
        Sigs1 = Sigs1 + (Phi^j)*RSe*(Phi^j)';
    end
    h = 4;
    Sigs4 = zeros(nx,nx);
    for j=0:h-1
        Sigs4 = Sigs4 + (Phi^j)*RSe*(Phi^j)';
    end
    h = 8;
    Sigs8 = zeros(nx,nx);
    for j=0:h-1
        Sigs8 = Sigs8 + (Phi^j)*RSe*(Phi^j)';
    end
    diff = 5;
    Sigsinf0 = zeros(nx,nx);
    j = 0;
    while diff > 1e-8
        Sigsinf1 = Sigsinf0 + (Phi^j)*RSe*(Phi^j)';
        diff=max(max(abs(Sigsinf1-Sigsinf0)));
        Sigsinf0 = Sigsinf1;
        j = j+1;
    end
    Sigs = Sigsinf1;
    % Inflation selector vector
    ipi = 1;
    epi = zeros(1,nx);
    epi(ipi) = 1;
    % Predictability, CPS (2010, eq. 12)
    Rsq1 = 1 - (epi*Sigs1*epi')/(epi*Sigs*epi');
    Rsq4 = 1 - (epi*Sigs4*epi')/(epi*Sigs*epi');
    Rsq8 = 1 - (epi*Sigs8*epi')/(epi*Sigs*epi');
end
