clear, close all
% Read data: 'GDPDEF.csv' contains the GDP Implicit Price Deflator,
% downloaded from FRB St Louis FRED on 11/10/2022
opts = detectImportOptions('GDPDEF.csv');
opts = setvaropts(opts,'observation_date','InputFormat','MM/dd/uuuu'); 
datain = readtable('GDPDEF.csv',opts);
gdpdef_pct_change = datain.GDPDEF(2:end)./datain.GDPDEF(1:end-1) - 1;
datain(1,:) = [];
datain.GDPDEF = gdpdef_pct_change;

%%%%%% PARMS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

p = 4; %set lags
splityear = 1984;
splitquarter = 4;
tablename = {'Table 2: US data'};
rownames = {'1959Q1-1984Q4','1985Q1-2019Q4','Difference'};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

data = {datain(1:103,:),datain(105:end,:)};

% Generate output table
EmpRes = table(zeros(size(rownames,2),1),zeros(size(rownames,2),1),zeros(size(rownames,2),1), zeros(size(rownames,2),1),'VariableNames',{'Sum AR(p)','R²₁','R²₄','R²₈'},'RowNames',rownames);
EmpRes = table(EmpRes,'VariableNames',tablename);

for d = 1:size(data,2)
       
        [alpha,res] = Pred_Compute(data{d}.(2),p);
        
        % Companion form matrix A
        A = diag(ones(1,p-1),-1);
        A(1,1:p) = alpha;
        coefsum = sum(alpha);
        %Covariance matrix of companion form
        R = [res, zeros(length(res),p-1)];
        V = R'*R;
        
        % Compute predictability score and sum autocorrelation coefficients
        [one,four,eight] = Predictabilityfun(A,V);
        EmpRes{d,:} = {sum(alpha),one,four,eight};
        EmpRes{size(data,2)+1,:} = num2cell(table2array(EmpRes{size(data,2),:}) - table2array(EmpRes{size(data,2)-1,:}));

end

% Display output
disp(EmpRes)

function [alpha,res] = Pred_Compute(data,p)
% Function that accepts a data matrix and lag instructions (see above) to
% later compute the predictability score
 
    t = size(data,1);

    % Initialize lag matrix (T-p)x(p)
    X = zeros(t-p,p);

    for j = 1:size(X,1)
        X(j,:) = fliplr(data(j:j+p-1)');
    
    end
    
    % Define Y vector, estimate coefficients/residuals and store them
    % Note that with fitlm, the X,Y input order is reversed relative to
    % regstats. I chose fitlm because it may eventually replace
    % regstats
    Y = data(p+1:end);
    mdl = fitlm(X,Y,'linear');
    res = mdl.Residuals.Raw;
    alpha = mdl.Coefficients.Estimate(2:end)';

end 

function [Rsq1,Rsq4,Rsq8] = Predictabilityfun(A,V)
% Function computes the predictability based on 
% Cogley, Timothy, Giorgio E. Primiceri, and Thomas J. Sargent. 2010. "Inflation-Gap Persistence in the US." 
    %=========================================================================
    % Companion form of AR(p) process: Y = A*Y(-1) + res
    % varres = res'*res
    n = size(A,1);
    % Approximate unconditional variance
    Sigy = dlyap_simple(A,1,V);
   %=========================================================================
    %       Predictability measure of inflation persistence
    %=========================================================================
    % Approximate forecast error variance, Cogley, Primiceri and Sargent (AEJ Macro, 2010, eq. 10)
    % horizon h = 1,4,8
    h = 1;
    Sigy1 = zeros(n,n);
    for j=0:h-1
        Sigy1 = Sigy1 + (A^j)*V*(A^j)';
    end
    h = 4;
    Sigy4 = zeros(n,n);
    for j=0:h-1
        Sigy4 = Sigy4 + (A^j)*V*(A^j)';
    end
    h = 8;
    Sigy8 = zeros(n,n);
    for j=0:h-1
        Sigy8 = Sigy8 + (A^j)*V*(A^j)';
    end
    % Inflation selector vector
    epi = zeros(1,n);
    epi(1) = 1;
    % Predictability, Cogley, Primiceri and Sargent (AEJ Macro, 2010, eq. 12)
    Rsq1 = 1 - (epi*Sigy1*epi')/(epi*Sigy*epi');
    Rsq4 = 1 - (epi*Sigy4*epi')/(epi*Sigy*epi');
    Rsq8 = 1 - (epi*Sigy8*epi')/(epi*Sigy*epi');
end